#!/bin/bash

# bwrep-sm.sh: Generate short bandwidth usage report.
# This script will retrieve the bandwidth monitoring page from theplanet
# hosting service and reformat it for a simple text display to IRC.
# Script includes an optional daemon mode <-d> which will update this
# text file every hour.
# To customise the scripts output look at the generate_output function.
# Note that script requires at least GNU wget 1.9.1.  1.8.x will not work as
# it lacks the --post-data option required to submit the username and pass.

# Global variables and starting values.
username='xxxx'
password='xxxx'
# This is the base directory for the script.
# Make sure you have write permissions for this directory.
bwrep_path="/jail/glftpd/sitebot/bwrep"
# These are filenames in the base directory:
output_file="$bwrep_path/bwrep.txt"
html_file="$bwrep_path/bwrep.tmp.html"
pid_file="$bwrep_path/bwrep.pid"
log_file="$bwrep_path/bwrep.log" # set to /dev/null to disable logging.
# URL constants:
url='https://orbit.theplanet.com'
# Initial values for program data variables.
total='0'
quota='0'
estimated='0'
daemon='0' # cmdline
ident='bwrep-sm.sh (V0.5) Tue May 24 13:07:16 EST 2005 DLP/X'

function log {
# Simple logging function.
echo "$*" >> $log_file
return 0
}
# End function.
function parse_stats {
# This function will parse the html input
# Takes one argument-the input file.
local parse_line
if test ! -r "$1"
then
log 'Error reading html input.'
exit 1
fi
local parse_input="$1"
# Now we retrieve the actual data and store it in the correct variables.
# This data will need to be sanity checked before writing the file.
parse_line=`grep 'Quick RAW Usage:' "$parse_input"`
total=`echo $parse_line |cut -d ':' -f2 |cut -d ' ' -f2`
reset_fields
parse_line=`grep 'Allowed Usage:' "$parse_input"`
quota=`echo $parse_line |cut -d ':' -f2 |cut -d ' ' -f2`
reset_fields
parse_line=`grep 'Estimated RAW Usage:' "$parse_input"`
estimated=`echo $parse_line |cut -d ':' -f2 |cut -d ' ' -f2`
reset_fields
log 'Parsed html file.'
return 0
} 
# End function.
function reset_fields {
# Reset the fields of the parser
# This helps with error checking if called correctly.
# helper function for parse_stats
parse_line=''
return 0
}
# End function.
function check_stats {
# This function sanity checks the data retrieved by parse_stats
# This function should always be called before writing out the final report.
if [ "$total" = '' -o "$total" = '0' ]
then
log 'Error retrieving total.'
return 1
fi
if [ "$quota" = '' -o "$quota" = '0' ]
then
log 'Error retrieving  quota.'
return 1
fi
if [ "$estimated" = '' -o "$estimated" = '0' ]
then
log 'Error retrieving estimate.'
return 1
fi
return 0
}
# End function.
function get_stats_data {
# This should retrieve the stats html from the web hosting company
rm -f "$html_file"
if ! touch "$html_file"
then
log 'Error writing to temp html file.'
exit 1
fi
if ! test -w "$html_file"
then
log 'Error writing to temp html file.'
return 1
fi
if wget -q -O $html_file \
--post-data "username=${username}&password=${password}&Submit=Submit" \
--save-headers $url/login.html \
$url/nav_hardware/a1_bandwidth_utilization.html
then
return 0
else
log 'wget returns error.'
return 1
fi
return 1
}
# End function
function generate_output {
# Generates the final report.
# this kills the old report so the data had better be sanity checked.
rm -f $output_file
if ! touch "$output_file"
then
 log 'Error writing to output file.'
return 1
fi
if test ! -w "$output_file"
then
log 'Error writing to output file.'
return 1
fi
echo "[DLP] ${total}GB of ${quota}GB has been used this month. \
If current usage rates are maintained ${estimated}GB will be used \
by the end of the month. \
These stats were last updated `date`.\
" > $output_file
 log 'Report generation complete.'
return 0
}
# End function.
function parse_cmdline {
# Parse command line options-only a few right now.
while [ "$1" != '' ]
do
if [ "$1" = '--help' -o "$1" = '-h' ]
then
usage_info
exit 0
elif [ "$1" = '--version' ]
then
 echo "$ident"
exit 0
elif [ "$1" = '-d' ]
then
daemon='1'
shift
else
echo "Invalid command line option: $1"
exit 1
fi
 done
return 0
} 
# End function.
function usage_info {
local prgname=`basename $0`
echo "$ident"
echo
echo "Usage: $prgname [-d]"
echo '-d: Run as a daemon and update every hour.'
exit 0
}
# End function.
function run_daemon {
# Runs this script as a daemon until sent a kill signal.
sleep 1
log 'Running as a daemon'
while test ! -e $bwrep_path/bwrep.exit
do
$0
log 'sleeping for 3600 seconds.'
sleep 3600
done
exit 0
}
## End function.

# Execution starts here
parse_cmdline $*
if [ "$daemon" = '1' ]
then
if [ "$log_file" != '/dev/null' ]
then
rm -f $log_file
fi
# first if we have a pid file try to kill off that old process.
if test -r "$pid_file"
then
old_pid=`cat "$pid_file"`
log "Trying to kill old process: $old_pid"
kill $old_pid
fi
rm -f $pid_file
run_daemon >& /dev/null &
echo "$!" > $pid_file
exit 0
fi
log "Program started `date`"
if ! get_stats_data
then
exit 1
fi
parse_stats "$html_file"
if check_stats
then
if generate_output
then
log 'success'
exit 0
else
log 'Error generating output.'
exit 1
 fi
else
log 'Error parsing html file.'
exit 1
fi
